--
-- unloadPlanes
--
-- @author:    	Xentro (www.fs-uk.com)(Marcus@Xentro.se)
-- @version:    v1.0
-- @date:       2013-04-15
-- 
--[[
<planeSets>
	<planeSet> <!-- possible to add more but you wont be able to change with this script -->
		<plane> <!-- normal -->
			<fillPlane type="" >
				<node index="">
					<key time="" translation="" />
				</node>
			</fillPlane>
		</plane>
		<plane> <!-- unloading -->
			<fillPlane type="" >
				<node index="">
					<key time="" translation="" />
				</node>
			</fillPlane>
		</plane>
	</planeSet>
</planeSets>
]]--

unloadPlanes = {};

unloadPlanes.UNLOADING = 2;
unloadPlanes.NORMAL = 1;

function unloadPlanes.prerequisitesPresent(specializations)
	if not SpecializationUtil.hasSpecialization(Trailer, specializations) then print("Warning: Specialization unloadPlanes needs the specialization Trailer."); end;
	if not SpecializationUtil.hasSpecialization(Fillable, specializations) then print("Warning: Specialization unloadPlanes needs the specialization Fillable."); end;
	
	return SpecializationUtil.hasSpecialization(Fillable, specializations) and SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function unloadPlanes:load(xmlFile)
	self.changePlaneSet = SpecializationUtil.callSpecializationsFunction("changePlaneSet");
	
	self.planeSets = {}
	local i = 0;
	while true do
		local key = string.format("vehicle.planeSets.planeSet(%d)", i);
		if not hasXMLProperty(xmlFile, key) then break; end;
		
		local entry = {};
		entry.plane = {};
		
		local planeI = 0;
		while true do
			local planeKey = key .. string.format(".plane(%d)", planeI);
			if not hasXMLProperty(xmlFile, planeKey) then break; end;
			local planeSet = {};
			planeSet.fillPlanes = {};
		
			if self.isClient then
				local fillI = 0;
				while true do
					local fillKey = planeKey .. string.format(".fillPlane(%d)", fillI);
					if not hasXMLProperty(xmlFile, fillKey) then break; end;
					
					local fillType = getXMLString(xmlFile, fillKey .. "#type");
					local fillPlane = {};
					fillPlane.nodes = {};
					
					if fillType ~= nil then
						local nodeI = 0;
						while true do
							local nodeKey = fillKey .. string.format(".node(%d)", nodeI);
							if not hasXMLProperty(xmlFile, nodeKey) then break; end;
							
							local node = Utils.indexToObject(self.components, getXMLString(xmlFile, nodeKey .. "#index"));
							if node ~= nil then
								local defaultX, defaultY, defaultZ = getTranslation(node);
								local defaultRX, defaultRY, defaultRZ = getRotation(node);
								setVisibility(node, false);
								
								local animCurve = AnimCurve:new(linearInterpolatorTransRotScale);
								local keyI = 0;
								while true do
									local animKey = nodeKey .. string.format(".key(%d)", keyI);
									if not hasXMLProperty(xmlFile, animKey) then break; end;
									
									local keyTime = getXMLFloat(xmlFile, animKey .. "#time");
									if keyTime ~= nil then
										local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, animKey .. "#translation"));
										if y == nil then
											y = getXMLFloat(xmlFile, animKey .. "#y");
										end;
										
										local rx, ry, rz = Utils.getVectorFromString(getXMLString(xmlFile, animKey .. "#rotation"));
										local sx, sy, sz = Utils.getVectorFromString(getXMLString(xmlFile, animKey .. "#scale"));
										local x = Utils.getNoNil(x, defaultX);
										local y = Utils.getNoNil(y, defaultY);
										local z = Utils.getNoNil(z, defaultZ);
										
										if fillPlanesRotDeg then
											rx = Utils.getNoNilRad(rx, defaultRX);
											ry = Utils.getNoNilRad(ry, defaultRY);
											rz = Utils.getNoNilRad(rz, defaultRZ);
										else
											rx = Utils.getNoNil(rx, defaultRX);
											ry = Utils.getNoNil(ry, defaultRY);
											rz = Utils.getNoNil(rz, defaultRZ);
										end
										
										local sx = Utils.getNoNil(sx, 1);
										local sy = Utils.getNoNil(sy, 1);
										local sz = Utils.getNoNil(sz, 1);
										
										animCurve:addKeyframe({x = x, y = y, z = z, rx = rx, ry = ry, rz = rz, sx = sx, sy = sy, sz = sz, time = keyTime});
										keyI = keyI + 1;
									else
										print("Error: key time is nil in '" .. self.configFileName .. "'");
										break;
									end;
								end;
								
								if keyI == 0 then
									local minY, maxY = Utils.getVectorFromString(getXMLString(xmlFile, nodeKey .. "#minMaxY"));
									local minY = Utils.getNoNil(minY, defaultY);
									local maxY = Utils.getNoNil(maxY, defaultY);
									
									animCurve:addKeyframe({x = defaultX, y = minY, z = defaultZ, rx = defaultRX, ry = defaultRY, rz = defaultRZ, sx = 1, sy = 1, sz = 1, time = 0});
									animCurve:addKeyframe({x = defaultX, y = maxY, z = defaultZ, rx = defaultRX, ry = defaultRY, rz = defaultRZ, sx = 1, sy = 1, sz = 1, time = 1});
								end;
								local alwaysVisible = Utils.getNoNil(getXMLBool(xmlFile, nodeKey .. "#alwaysVisible"), false);
								
								table.insert(fillPlane.nodes, {node = node, animCurve = animCurve, alwaysVisible = alwaysVisible});
								nodeI = nodeI + 1;
							else
								print("Error: node index is nil in '" .. self.configFileName .. "'");
								break;
							end;
						end;
						
						if table.getn(fillPlane.nodes) > 0 then
							if self.defaultFillPlane == nil then
								self.defaultFillPlane = fillPlane;
							end;
							planeSet.fillPlanes[fillType] = fillPlane;
						end;
						
						fillI = fillI + 1;
					end;
				end;
			end;
			
			table.insert(entry.plane, planeSet);
			planeI = planeI + 1;
		end;
			
		if table.getn(entry.plane) == 2 then
			table.insert(self.planeSets, entry);
		else
			print("Warning: there must be 2 <plane> (normal and unloading) for each planeSet in " .. self.configFileName);
		end;
		
		i = i + 1;
	end;	
	
	self.currentPlaneSetId = 1;
	self.currentPlaneId = unloadPlanes.NORMAL;
	
	self.maxPlanes = table.getn(self.planeSets);
	
	self.sentPlaneSetId = self.currentPlaneSetId;
	self.sentPlaneId = self.currentPlaneId;
	
	self.allowAutoChange = self.maxPlanes > 0;
	
	if self.allowAutoChange then
		self:changePlaneSet(self.currentPlaneSetId);
	end;
end;

function unloadPlanes:delete()
end;

function unloadPlanes:readStream(streamId, connection)
	self:changePlaneSet(streamReadInt8(streamId), true);
end;

function unloadPlanes:writeStream(streamId, connection)
	streamWriteInt8(streamId, self.currentPlaneSetId);
end;

function unloadPlanes:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local currentPlaneSetId = getXMLInt(xmlFile, key .. "#currentPlaneSetId");
	if currentPlaneSetId ~= nil then
		self:changePlaneSet(currentPlaneSetId, true);
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;
 
function unloadPlanes:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'currentPlaneSetId="' .. self.currentPlaneSetId .. '"';
	return attributes, nil;
end;

function unloadPlanes:mouseEvent(posX, posY, isDown, isUp, button)
end;

function unloadPlanes:keyEvent(unicode, sym, modifier, isDown)
end;

function unloadPlanes:update(dt)
	if self.isServer then
		if self.currentPlaneId ~= self.sentPlaneId or self.currentPlaneSetId ~= self.sentPlaneSetId then
			self:changePlaneSet(self.currentPlaneSetId);
			
			self.sentPlaneSetId = self.currentPlaneSetId;
			self.sentPlaneId = self.currentPlaneId;
		end;
		
		if self.allowAutoChange then
			if self.tipState ~= Trailer.TIPSTATE_CLOSED then
				if self.currentPlaneId ~= unloadPlanes.UNLOADING then
					self.currentPlaneId = unloadPlanes.UNLOADING;
				end;
			else
				if self.currentPlaneId ~= unloadPlanes.NORMAL then
					self.currentPlaneId = unloadPlanes.NORMAL;
				end;
			end;
		end;
	end;
end;

function unloadPlanes:updateTick(dt)
end;

function unloadPlanes:draw()
end;

function unloadPlanes:changePlaneSet(id, noEventSend)	
	if id > self.maxPlanes or id <= 0 then
		id = 1;
	end;
	
	local planeSet = self.planeSets[id];
	
	if planeSet ~= nil then
		local plane = planeSet.plane[self.currentPlaneId];
		
		if plane ~= nil then		
			self.fillPlanes = plane.fillPlanes;
		end;
		
		self:setFillLevel(self.fillLevel, self.currentFillType);
	end;
	
	self.currentPlaneSetId = id;
	setPlaneEvent.sendEvent(self, id, noEventSend);
end;


setPlaneEvent = {};
setPlaneEvent_mt = Class(setPlaneEvent, Event);

InitEventClass(setPlaneEvent, "setPlaneEvent");

function setPlaneEvent:emptyNew()
    local self = Event:new(setPlaneEvent_mt);
    self.className = "setPlaneEvent";
    return self;
end;

function setPlaneEvent:new(object, id)
    local self = setPlaneEvent:emptyNew()
    self.object = object;
	self.id = id;
    return self;
end;

function setPlaneEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.id = streamReadInt8(streamId);
    self.object = networkGetObject(id);
    self:run(connection);
end;

function setPlaneEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteInt8(streamId, self.id);
end;

function setPlaneEvent:run(connection)
	self.object:changePlaneSet(self.id, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(setPlaneEvent:new(self.object, self.id), nil, connection, self.object);
	end;
end;

function setPlaneEvent.sendEvent(object, id, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setPlaneEvent:new(object, id), nil, nil, object);
		else
			g_client:getServerConnection():sendEvent(setPlaneEvent:new(object, id));
		end;
	end;
end;